# Copyright (C) 2009-2010 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

# -*- coding: utf-8 -*-
"""Cluster controller module"""

# turbogears imports
from tg import request, expose, redirect, flash, app_globals, tmpl_context

# third party imports
from repoze.what.predicates import not_anonymous 
from pylons.i18n import ugettext as _

# project specific imports
from luci.lib.base import BaseController
from luci.lib.db_helpers import get_cluster_list, get_model_for_cluster, get_status_for_cluster, db_remove_cluster, get_agent_for_cluster
import luci.lib.ricci_helpers as rh
import luci.widget_validators.validate_cluster_prop as vcp
from luci.widget_validators.validate_create_cluster_form import validate_create_cluster_form 
from luci.widget_validators.validate_fence import validateNewFenceDevice, validateFenceDevice
from luci.widget_validators.validate_resource import create_resource, validate_clusvc_form 

import logging
log = logging.getLogger(__name__)

data = app_globals.data

class ClusterController(BaseController):
    allow_only = not_anonymous()
    tmpl_context.show_sidebar = True

    @expose('luci.templates.cluster_list')
    def index(self):
        tmpl_context.manage_clusters = True
        tmpl_context.show_sidebar = True
        return dict(page='cluster', clusterpage='clusterpage')

    @expose('luci.templates.cluster_list')
    def create_cmd(self, command=None, **kw):
        validate_create_cluster_form(self, **kw)
        return dict(use_referrer=False, page='cluster', clusterpage='clusterpage')

    @expose('luci.templates.cluster_list')
    def delete_cmd(self, command=None, **kw):
        cur_list = []
        if 'MultiAction' in kw:
            command = kw['MultiAction']
            for key, element in kw.items():
                if element == "on":
                     db_remove_cluster(key)

        redirect('/cluster/')

    @expose()
    def lookup(self, name, *args):
        cluster_list = get_cluster_list() or []
        for i in cluster_list:
            if i.name == name:
                icc = IndividualClusterController(name, data)
                return icc, args
        cc = ClusterController
        #flash(("No cluster named \"%s\" is managed by luci") % name,
        #    status='error')
        redirect('/cluster/')
        return (cc, args)

class IndividualClusterController(BaseController):
    def __init__(self, name, iccdata, nodename=None):
        self.name = name
        self.data = iccdata
        self.nodename = nodename
        identity = request.environ.get('repoze.who.identity')
        self.username = identity['repoze.who.userid']
        tmpl_context.show_sidebar = True

    @expose("luci.templates.node")
    def default(self):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name
        return dict(page='nodes', name=self.nodename, base_url='/nodes')

    @expose("luci.templates.node")
    def nodes(self):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name
        return dict(use_referrer=False, page='nodes', name=self.nodename, base_url='/nodes')

    @expose("luci.templates.node")
    def add_nodes_cmd(self, command=None, **kw):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name
        return dict(use_referrer=False, page='nodes', name=self.nodename, base_url='/nodes')

    @expose("luci.templates.node")
    def nodes_fence_cmd(self, command=None, **kw):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name
        
        if not cmodel:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)

        vret = vcp.validate_node_fence_form(cmodel, **kw)
        if vret[0] is True:
            log.info('User "%s" updated the fence properties for cluster "%s" node "%s"'
                % (self.username, self.name, kw.get('node')))
            flash(_('Updating fence settings'))
            msgs = vret[1].get('flash')
            if msgs:
                flash(', '.join(msgs))
            cmodel.setModified(True)
            rh.update_cluster_conf(cmodel)
        else:
            msgs = vret[1].get('errors')
            if msgs and len(msgs) > 0:
                flash(', '.join(msgs), status="error")
        redirect('%s%s' % (tmpl_context.cluster_url, kw.get('node')))
        
    # This processes all of the commands that we can apply to a node
    @expose("luci.templates.node")
    def nodes_cmd(self, command=None, **kw):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name

        if not cmodel:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)

        cur_list = []
        if 'MultiAction' in kw:
            kw["name"] = ""
            command = kw['MultiAction']
            cur_list = [x for x in kw if kw[x] == 'on' in kw[x]]
        else:
            obj_name = kw.get('name')
            if obj_name:
                cur_list = [ obj_name ]
        if len(cur_list) < 1:
            flash(_('No nodes were selected'),
                status='error')
            redirect(tmpl_context.cluster_url)

        if command == 'Reboot':
            log.info('User "%s" rebooted "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Rebooting %s") % ', '.join(cur_list),
                status='info')
            rh.cluster_node_reboot(self.name, cur_list)
        elif command == 'Join Cluster':
            log.info('User "%s" started nodes "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Starting nodes: %s") % ', '.join(cur_list),
                status='info')
            rh.cluster_node_start(self.name, cur_list)
        elif command == 'Leave Cluster':
            log.info('User "%s" stopped nodes "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Stopping nodes: %s") % ', '.join(cur_list),
                status='info')
            rh.cluster_node_stop(self.name, cur_list)
        elif command == 'Delete':
            log.info('User "%s" deleted nodes "%s" from cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Deleting nodes: %s") % ', '.join(cur_list),
                status='info')
            rh.cluster_node_delete(self.name, cur_list)
        else:
            log.error('User "%s" submitted unknown command "%s" for nodes "%s" from cluster "%s"' % (self.username, command, ', '.join(cur_list), self.name))
            flash(_('An unknown command "%s" was given for nodes "%s"')
                    % (command, ', '.join(cur_list)),
                status='error')
        redirect(tmpl_context.cluster_url + kw["name"])

    @expose("luci.templates.resource")
    def resources(self, *args):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name

        if (len(args) == 1):
            resourcename = args[0]
        else:
            resourcename = None

        base_url = '/cluster/%s/resources' % self.name
        resources_cmd = '/cluster/%s/resources_cmd' % self.name
        return dict(page='nodes', name=resourcename, base_url=base_url, resources_cmd=resources_cmd)

    @expose("luci.templates.resource")
    def resources_cmd(self, command=None, **kw):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/resources' % self.name
        
        if not cmodel:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)

        cur_list = []
        if 'MultiAction' in kw:
            kw["name"] = ""
            command = kw['MultiAction']
            cur_list = [x for x in kw if kw[x] == 'on' in kw[x]]
        else:
            obj_name = kw.get('name')
            if obj_name:
                cur_list = [ obj_name ]

        if len(cur_list) < 1 and command != 'Create':
            flash(_('No resources were selected'),
                status='error')
            redirect(tmpl_context.cluster_url)
        if command == 'Delete':
            log.info('User "%s" deleted resource "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Deleting resources %s") % ', '.join(cur_list),
                status='info')
            for i in cur_list:
                cmodel.deleteResource(i)
            cmodel.setModified()
            rh.update_cluster_conf(cmodel)
        elif command == "Create":
            res = create_resource(kw["type"], cmodel, **kw)
            cmodel.getResourcesPtr().addChild(res)
            cmodel.setModified(True)
            rh.update_cluster_conf(cmodel)
            redirect(tmpl_context.cluster_url)
        else:
            log.error('User "%s" submitted unknown command "%s" for resource "%s" from cluster "%s"' % (self.username, command, ', '.join(cur_list), self.name))
            flash(_('An unknown command "%s" was given for resources "%s"')
                % (command, ', '.join(cur_list)),
                    status='error')
        redirect(tmpl_context.cluster_url)

    @expose("luci.templates.service")
    def services(self, *args):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name

        if (len(args) == 1):
            servicename = args[0]
        else:
            servicename = None

        base_url = '/cluster/%s/services' % self.name
        services_cmd = '/cluster/%s/services_cmd' % self.name
        return dict(page='nodes', name=servicename, base_url=base_url, services_cmd=services_cmd)

    @expose("luci.templates.service")
    def services_cmd(self, command=None, **kw):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/services' % self.name

        if not cmodel:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)
        
        preferred_node = kw.get('preferred_node')

        cur_list = []
        if 'MultiAction' in kw:
            kw["name"] = ""
            command = kw['MultiAction']
            cur_list = [x for x in kw if kw[x] == 'on' in kw[x]]
        else:
            obj_name = kw.get('name')
            if obj_name:
                cur_list = [ obj_name ]

        if len(cur_list) < 1 and command not in ('Create', 'Edit'):
            flash(_('No services were selected'),
                status='error')
            redirect(tmpl_context.cluster_url)

        if command == 'Delete':
            log.info('User "%s" deleted service "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Deleting services %s") % ', '.join(cur_list),
                status='info')
            for i in cur_list:
                cmodel.deleteService(i)

            cmodel.setModified()
            rh.update_cluster_conf(cmodel)
            redirect(tmpl_context.cluster_url)
        if command == 'Start':
            log.info('User "%s" started service "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Starting services %s") % ', '.join(cur_list),
                status='info')
            for i in cur_list:
                rh.cluster_svc_start(self.name, i, preferred_node)
        elif command == 'Disable':
            log.info('User "%s" disabled service "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Disabling services %s") % ', '.join(cur_list),
                status='info')
            for i in cur_list:
                rh.cluster_svc_disable(self.name, i)
        elif command == 'Restart':
            log.info('User "%s" restarted service "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Restarting services %s") % ', '.join(cur_list),
                status='info')
            for i in cur_list:
                rh.cluster_svc_restart(self.name, i)
        elif command in ('Create', 'Edit'):
            if command == 'Create':
                cur_action = _('Created')
            else:
                cur_action = _('Edited')
            vret = validate_clusvc_form(cmodel, **kw)
            kw['name'] = ''
            if vret[0] is True:
                svc_name = kw.get('svc_name')
                log.info('User "%s" %s cluster service "%s" in cluster "%s"'
                    % (self.username, cur_action, svc_name, self.name))
                flash(_('%s cluster service "%s"') % (cur_action, svc_name))
                cmodel.setModified(True)
                rh.update_cluster_conf(cmodel)
            else:
                msgs = vret[1].get('errors')
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
            redirect(tmpl_context.cluster_url)
        else:
            log.error('User "%s" submitted unknown command "%s" for service "%s" from cluster "%s"' % (self.username, command, ', '.join(cur_list), self.name))
            flash(_('An unknown command "%s" was given for services "%s"')
                    % (command, ', '.join(cur_list)),
                status='error')
        if len(cur_list) != 1:
            redirect(tmpl_context.cluster_url)
        else:
            redirect(tmpl_context.cluster_url + '/' + cur_list[0])

    @expose("luci.templates.qdisk")
    def qdisk(self, *args):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name
        qdisk_cmd = '/cluster/%s/qdisk_cmd' % self.name
        return dict(page='nodes', base_url = '/cluster/' + self.name + '/qdisk', qdisk_cmd = qdisk_cmd)

    @expose("luci.templates.qdisk")
    def qdisk_cmd(self, command=None, **kw):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name
        qdisk_cmd = '/cluster/%s/qdisk_cmd' % self.name

        if not cmodel:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            return dict(use_referrer=False, page='nodes', base_url = '/cluster/' + self.name + '/qdisk', qdisk_cmd=qdisk_cmd)

        vret = vcp.validate_qdisk_prop_form(cmodel, **kw)
        if vret[0] is True:
            log.info('User "%s" updated the quorum disk properties for cluster "%s"'
                        % (self.username, self.name))
            flash(_('Updating quorum disk settings'))
            msgs = vret[1].get('flash')
            if msgs:
                flash(', '.join(msgs))

            cmodel.setModified(True)
            rh.update_cluster_conf(cmodel)
        else:
            msgs = vret[1].get('errors')
            if msgs and len(msgs) > 0:
                flash(', '.join(msgs), status="error")
        return dict(use_referrer=False, page='nodes', base_url = '/cluster/' + self.name + '/qdisk', qdisk_cmd=qdisk_cmd)

    @expose("luci.templates.failover")
    def failovers(self, *args):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name
        if len(args) == 1:
            failovername = args[0]
        else:
            failovername = None
        failovers_cmd = '/cluster/%s/failovers_cmd' % self.name
        return dict(page='nodes', name=failovername, base_url = '/cluster/' + self.name + '/failovers', failovers_cmd=failovers_cmd)

    @expose("luci.templates.failover")
    def failovers_cmd(self, command=None, **kw):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/failovers' % self.name

        if not cmodel:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)

        cur_list = []
        if 'MultiAction' in kw:
            kw["name"] = ""
            command = kw['MultiAction']
            cur_list = [x for x in kw if kw[x] == 'on' in kw[x]]
        else:
            obj_name = kw.get('name')
            if obj_name:
                cur_list = [ obj_name ]

        if len(cur_list) < 1 and command != 'create':
            flash(_('No failover domains were selected'),
                status='error')
            redirect(tmpl_context.cluster_url)

        if command == 'Delete':
            log.info('User "%s" deleted failover domains "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_('Deleting failover domains %s') % ', '.join(cur_list))
            for i in cur_list:
                cmodel.deleteFailoverDomain(i)
            cmodel.setModified(True)
            rh.update_cluster_conf(cmodel)

            redirect(tmpl_context.cluster_url)

        # Delete is the only command that can be applied to multiple
        # failover domains
        if len(cur_list) > 1:
            flash(_('The settings of exactly one failover domain may be updated at one time'),
                status='error')
            redirect(tmpl_context.cluster_url)
        
        if command == 'update_settings':
            vret = vcp.validate_fdom_prop_settings_form(cmodel, **kw)
            if vret[0] is True:
                log.info('User "%s" updated the settings of failover domain "%s" in cluster "%s"'
                    % (self.username, cur_list[0], self.name))
                flash(_('Updating settings for failover domain "%s"') % cur_list[0])
                cmodel.setModified(True)
                rh.update_cluster_conf(cmodel)
            else:
                msgs = vret[1].get('errors')
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
            redirect(tmpl_context.cluster_url + '/' + kw["name"])
        elif command == 'update_properties':
            vret = vcp.validate_fdom_prop_form(cmodel, **kw)
            if vret[0] is True:
                log.info('User "%s" updated the properties of failover domain "%s" in cluster "%s"'
                    % (self.username, cur_list[0], self.name))
                flash(_('Updating properties for failover domain "%s"') % cur_list[0])
                cmodel.setModified(True)
                rh.update_cluster_conf(cmodel)
            else:
                msgs = vret[1].get('errors')
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
            redirect(tmpl_context.cluster_url + '/' + kw["name"])
        elif command == 'create':
            vret = vcp.validate_fdom_create_form(cmodel, **kw)
            if vret[0] is True:
                log.info('User "%s" created failover domain "%s" in cluster "%s"'
                    % (self.username, kw.get('fdom_name'), self.name))
                flash(_('Creating failover domain "%s"') % kw.get('fdom_name'))

                cmodel.setModified(True)
                rh.update_cluster_conf(cmodel)
            else:
                msgs = vret[1].get('errors')
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
            redirect(tmpl_context.cluster_url)
        else:
            log.error('User "%s" submitted unknown command "%s" for failover domains "%s" from cluster "%s"'
                % (self.username, command, ', '.join(cur_list), self.name))
            flash(_('An unknown command "%s" was given for failover domains "%s"')
                    % (command, ', '.join(cur_list)),
                status='error')
        redirect(tmpl_context.cluster_url)

    @expose("luci.templates.fence")
    def fences(self, *args):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name

        if len(args) == 1:
            fencename = args[0]
        else:
            fencename = None

        fences_cmd = '/cluster/%s/fences_cmd' % self.name
        return dict(page='nodes', name=fencename, base_url='/cluster/' + self.name + '/fences', fences_cmd=fences_cmd)

    @expose("luci.templates.fence")
    def fences_cmd(self, command=None, **kw):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/fences' % self.name

        if not cmodel:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)

        cur_list = []
        if 'MultiAction' in kw:
            kw["name"] = ""
            command = kw['MultiAction']
            cur_list = [x for x in kw if kw[x] == 'on' in kw[x]]
        else:
            obj_name = kw.get('name')
            if obj_name:
                cur_list = [ obj_name ]

        if len(cur_list) < 1:
            flash(_('No fence devices were selected'),
                status='error')
            redirect(tmpl_context.cluster_url)

        if command == 'Create':
            fret = validateNewFenceDevice(cmodel, **kw)
            if fret[0] is True:
                cmodel.setModified(True)
                rh.update_cluster_conf(cmodel)
                log.info('User "%s" created fence devices "%s" in cluster "%s"'
                    % (self.username, ', '.join(cur_list), self.name))
            else:
                msgs = fret[1]
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
        elif command == 'Delete':
            log.info('User "%s" deleted fence devices "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_('Deleting fence devices ') + ', '.join(cur_list))
            updated = False
            for i in cur_list:
                updated |= cmodel.deleteFenceDevice(i)
            if updated:
                cmodel.setModified()
                rh.update_cluster_conf(cmodel)
        elif command == 'Update':
            fret = validateFenceDevice(cmodel, **kw)
            if fret[0] is True:
                cmodel.setModified(True)
                rh.update_cluster_conf(cmodel)
                log.info('User "%s" created fence device "%s" in cluster "%s"'
                    % (self.username, ', '.join(cur_list), self.name))
            else:
                msgs = fret[1]
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
        else:
            log.error('User "%s" submitted unknown command "%s" for fence devices "%s" from cluster "%s"'
                % (self.username, command, ', '.join(cur_list), self.name))
            flash(_('An unknown command "%s" was given for fence devices "%s"')
                    % (command, ', '.join(cur_list)),
                status='error')
        redirect(tmpl_context.cluster_url)

    @expose("luci.templates.configure")
    def configure(self, *args):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name
        configure_cmd = '/cluster/%s/configure_cmd' % self.name
        return dict(use_referrer=False, page='nodes', name='configure', base_url='/cluster/' + self.name + '/configure', configure_cmd=configure_cmd)

    @expose("luci.templates.configure")
    def configure_cmd(self, command=None, *args, **kw):
        try:
            rc = get_agent_for_cluster(self.name)
        except:
            rc = None
        cmodel = get_model_for_cluster(self.name, rc)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name, rc)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/configure' % self.name

        if not cmodel:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)

        vret = vcp.validate_cluster_config_form(cmodel, **kw)
        if vret[0] is True:
            log.info('User "%s" updated the cluster properties for cluster "%s"'
                    % (self.username, self.name))
            flash("Applying Settings")
            cmodel.setModified(True)
            rh.update_cluster_conf(cmodel)
        else:
            msgs = vret[1].get('errors')
            if msgs and len(msgs) > 0:
                flash(', '.join(msgs), status="error")
        redirect(tmpl_context.cluster_url)

    @expose()
    def lookup(self, nodename, *args):
        clustername = self.name
        cmodel = get_model_for_cluster(self.name)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name)
        tmpl_context.cluster_name = clustername
        tmpl_context.cluster_url = '/cluster/%s/' % clustername
        inc = IndividualNodeController(clustername, self.data, nodename)
        return inc, args

class IndividualNodeController(BaseController):
    def __init__(self, name, iccdata, nodename):
      self.name = name
      self.data = iccdata
      self.nodename = nodename
      tmpl_context.show_sidebar = True

    @expose("luci.templates.node")
    def default(self):
        cmodel = get_model_for_cluster(self.name)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name)
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name
        return dict(page='nodes', name=self.nodename, base_url='/nodes')

    @expose()
    def lookup(self, nodename, *args):
        clustername = self.name
        cmodel = get_model_for_cluster(self.name)
        tmpl_context.cluster_data = cmodel
        tmpl_context.cluster_status = get_status_for_cluster(self.name)
        tmpl_context.cluster_name = clustername
        icc = IndividualClusterController(clustername, self.data, nodename)
        return icc, args
